/*
	File:		SortPicts.cp

	Contains:	MPSortPicts a demo of MP Tasking

	Written by: Matthew Xavier Mora	

	Copyright:	Copyright  1996-1999 by Apple Computer, Inc., All Rights Reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):
				7/27/1999	Karl Groethe	Updated for Metrowerks Codewarror Pro 2.1
				1/20/97		Xavier Mora		to show MP sorting and blitting preemptively

*/
#define BUILDING_FOR_SYSTEM7_AND_SYSTEM8 1

#include "MPBlitter.h"
#include "SortPicts.h"
//#include "StandardMenus.h"
#include <string.h>
#include <strings.h>
#include <Drag.h>
#include <LowMem.h>
#include <Sound.h>
#include <Menus.h>

long		SortPicts::gNumWindowsCreated = 0;
short		SortPicts::gPrefsRef = 0;
short		gSortAlgorithm = 1;
long		gNumSortPictResources = 0;
MenuHandle	gPicturesMenu = 0;
Boolean		gUseCoplandTasks = false;
short		gTaskNumber = 1;

pascal short MyMBarHook(Rect *menuRect);
pascal short MyMBarHook(Rect *menuRect)
{
#pragma unused(menuRect)
	// should check and see if menurect intersects any of our blitters
	MPYield();
	return 0;
}
pascal void MyMenuHook(void);
pascal void MyMenuHook(void)
{
	MPYield();
}

SortPicts::SortPicts()
{
	MenuHandle	sortAlgorithmMenu;
	coplandTask = gUseCoplandTasks;
	updateTime = TickCount();
	lastHorizLeft = 0;
	lastHorizRight = 0;
	
	randomSeed = TickCount() & 0x7FFFFFFF;


	//LMGetMBarHook_UniversalProcPtr;
	LMSetMBarHook(NewMBarHookProc(MyMBarHook));
	LMSetMenuHook(NewMenuHookProc(MyMenuHook));
	if( gPrefsRef == 0)
	{
		sortAlgorithmMenu = GetMenuHandle( mSortAlgorithm);
		if( sortAlgorithmMenu)
		{
			CheckItem( sortAlgorithmMenu, gSortAlgorithm, true);
			CheckItem( sortAlgorithmMenu, iUseCoplandTasks, gUseCoplandTasks);
		}
	
		gPicturesMenu = GetMenuHandle( mPictureMenu);
		if( gPicturesMenu)
		{
			AppendResMenu( gPicturesMenu, 'PICT');
			InsertMenu( gPicturesMenu, -1);
		
		//	Count the number of Pictures in the menu
			gNumSortPictResources = CountMItems( gPicturesMenu);
		}

		gPrefsRef = OpenPreferencesResFile();
		if( gPrefsRef > 0)
		{
			OpenDefaultSortWindows();
		}
	}
	else
	{
		SelectRandomPicture();
		CreateWindow( kNormalWindow);
	}
}

SortPicts::SortPicts( ConstStr255Param windowName)
{
	randomSeed = TickCount() & 0x7FFFFFFF;
	coplandTask = gUseCoplandTasks;
	updateTime = TickCount();
	lastHorizLeft = 0;
	lastHorizRight = 0;

	::strcpy( (char *) sortPictsName, (char *) windowName);

	CreateWindow( kNormalWindow);
}

SortPicts::~SortPicts()
{
/*	Terminate Threads	*/

	if ( ! coplandTask ) {
		DisposeThread( threadInfo, nil, true);
	} else {
		OSStatus	status = 0;
		
		//  kill the copland task.
		//TerminateTask( myTaskID, kTaskOnly, 0, status );
		MPTerminateTask( myTaskID,status );
		UnuseSortData();
	}
	
/*	Deallocate Memory	*/
	ReleaseSortData();
	DisposeGWorld( sortGWorld);
}




WindowPtr SortPicts::MakeNewWindow(WindowPtr /* behindWindow */) 
{
	me = nil;
	
	if( PrepareGWorld( (ConstStr255Param) sortPictsName)) {

		if  (coplandTask) {
			p2cstr(sortPictsName);
			strcat((char*)sortPictsName,"(MP)");
			c2pstr((char*)sortPictsName);
		}
		me = NewCWindow( nil, &windPictRect, (ConstStr255Param) sortPictsName,
					true, 0, (WindowPtr) -1, true, (long) this);
		
		MakeThreaded();
		
	}
		
	return me;
}


void
SortPicts::Activate(Boolean /* activating */)
{
//	DrawGrowIcon(fWindow);
}


void
SortPicts::AdjustCursor(EventRecord * /* anEvent */)
{
}


void
SortPicts::Draw(void)
{
	

	
	if ( coplandTask ) {
		//CopyFromSortData();
		MPCopyBits( ( BitMap *) *sortPixmap, 
			  ( BitMap *) (*(((CWindowPtr)me)->portPixMap)),
			  &sortRect,&sortRect,srcCopy,(RgnHandle)sortBlitterBusy); 

	} else {
		sortPixmap = GetGWorldPixMap( sortGWorld);
		LockPixels( sortPixmap );

		CopyBits( (const BitMap *) *sortPixmap, 
			  (const BitMap *) (*(((CWindowPtr)me)->portPixMap)),
			  &sortRect,&sortRect,srcCopy,nil);
		UnlockPixels( sortPixmap );
	}
}


// I modified the drag method to do the Live dragging
void
SortPicts::	Drag(Point startPoint)
{


	Rect r = {0,0,0,0};
	Point origin = {0,0};	
	Point diff;
	EventRecord event;
	GrafPtr tempPort;
	KeyMap		theKeyMap;
	Boolean 	gLiveDrag;
Boolean		commandKeyDown = false;

	GetKeys(theKeyMap);
	commandKeyDown = ((theKeyMap[1] & 0x8000) != 0);
	gLiveDrag = (theKeyMap[1] >> 2) & 0x01 ;
	
	if (gLiveDrag || coplandTask) {	
		GetPort(&tempPort);
		SetPort(fWindow);	
		(void)OSEventAvail(everyEvent,&event);
		
		LocalToGlobal(&origin);
		diff.h = ( origin.h - event.where.h);
		diff.v = ( origin.v -event.where.v);

		SetPort(tempPort);
		while (StillDown()) {
			EventRecord event;
			(void)WaitNextEvent(everyEvent,&event,1,nil);
			
			if (event.what == updateEvt) {
				WindowRef window = (WindowRef)event.message;
				
				
				if (window) {
					TWindow * wobj;
					
					SetPort(window);
					BeginUpdate(window);
					wobj = GetWindowObject(window);

					wobj->Draw();
					EndUpdate(window);
					SetPort(tempPort);
				}
			}
			MPYield(); // for single CPU Macs
			event.where.h += diff.h ;
			event.where.v += diff.v;

			if (coplandTask) { // don't move the window if the blitter is blasting bits on the screen
				(void)MPEnterCriticalRegion(sortBlitterBusy, kDurationForever);
			}
			MoveWindow(fWindow,event.where.h,event.where.v,true);
			if (coplandTask) {
				(void)MPExitCriticalRegion(sortBlitterBusy);
			}
			Idle(nil);
		}
	}
	
	TWindow::Drag(startPoint); // call the real drag
	if (!coplandTask) {
		Idle(nil);
	}
	
}

	
void
SortPicts::Idle(EventRecord * /* anEvent */)
{



	if (  LMGetTicks() > updateTime )
	{
		UpdateSortPict();
		updateTime = LMGetTicks() + kUpdateInterval; 
		// when you're in macsbug LMTicks doesn't get updated
		// so the sorting animation doesn't happen. But the 
		// MPTask still sorts and blits the result to the screen
		// even when in macsbug. Cool!
		// I should probably use Uptime here

	} 

	


}


void
SortPicts::KeyDown(EventRecord * /* anEvent */ )
{
	this->Select();
}


void
SortPicts::AdjustForNewWindowSize(Rect * /* oldSize */, Rect * /* newSize */)
{
//	DrawGrowIcon(fWindow);	
}


Boolean
SortPicts::Close(void)
{
		
	DisposeThread( threadInfo, nil, true);
	return TWindow::Close();
}


OSErr
SortPicts::HandleDrag(DragTrackingMessage dragMessage,DragReference theDrag)
{
	RgnHandle	hiliteRgn = NewRgn();
	
	switch (dragMessage)
	{
		case kDragTrackingEnterWindow:
			SetRectRgn( hiliteRgn,
						fWindow->portRect.left,
						fWindow->portRect.top,
						fWindow->portRect.right,
						fWindow->portRect.bottom);
			ShowDragHilite(theDrag,hiliteRgn,true);
			break;
			
		case	kDragTrackingLeaveWindow:
			HideDragHilite(theDrag);
			break;
			
		default:
			break;
	}
	
	DisposeRgn(hiliteRgn);
	return(noErr);
}

	
OSErr
SortPicts::HandleDrop(DragReference /* theDrag */)
{
	return(noErr);
}
extern long			gNumSortPictResources;
extern MenuHandle	gPicturesMenu;
extern Boolean		gUseCoplandTasks;

Boolean	SortPicts::PrepareGWorld( ConstStr255Param pictName)
{

	if( !LoadSortPicture( pictName))
		return false;

	if( !MakeSortGWorld())
	{
		ReleaseSortPicture();
		return false;
	}

	ReleaseSortPicture();

	if( !AllocSortData())
	{
		DisposeGWorld( sortGWorld);
		return false;
	}
	
	MakeSortData();
	InitializeSortWindowRect( pictName);
	
	return true;			//	YES!! The everything worked out!
}

Boolean	SortPicts::LoadSortPicture( ConstStr255Param pictName)
{
	sortPict = (PicHandle)GetNamedResource( 'PICT', pictName);
	
	if( sortPict == nil)
		return false;
	
	sortRect = (*sortPict)->picFrame;
	pictWidth = sortRect.right - sortRect.left;
	pictHeight = sortRect.bottom - sortRect.top;
	
	SetRect(   &sortRect, 0, 0, (short) pictWidth, (short) pictHeight);
	copyBitsRect = sortRect;
	
	return true;
}

void	SortPicts::ReleaseSortPicture( void)
{
	ReleaseResource( (Handle) sortPict);

	sortPict = nil;
}

Boolean	SortPicts::MakeSortGWorld( void)
{
	OSErr			myOSErr;
	GWorldPtr		tempSortGWorld;
	GDHandle		oldGDevice;
	CGrafPtr		oldGWorld;
	Rect			tempSortRect;

	tempSortRect = sortRect;

	GetGWorld( &oldGWorld, &oldGDevice);
	
	myOSErr = NewGWorld( &tempSortGWorld, 8, &tempSortRect, 
						(CTabHandle) 0, (GDHandle) 0, keepLocal);
	
	if( myOSErr != noErr)
		return false;
			
	sortGWorld = tempSortGWorld;
	
	SetGWorld( sortGWorld, (GDHandle)0);
	sortPixmap = GetGWorldPixMap( sortGWorld );
	LockPixels( sortPixmap);

	
	EraseRect( &sortRect);
	DrawPicture( sortPict, &sortRect);
	
	UnlockPixels( sortPixmap );
	SetGWorld( oldGWorld, oldGDevice);

	return true;	
}


Boolean	SortPicts::AllocSortData( void)
{
	sortPixmap = GetGWorldPixMap( sortGWorld);

	pictRowBytes = (*sortPixmap)->rowBytes & 0x3FFF;

	N = pictWidth * pictHeight;
	
	sortHandle = (SortDataHandle) NewHandle( N * sizeof(long) );
				
	if( sortHandle == nil)
		return false;
	
	if ( gUseCoplandTasks )
	{
		HLockHi( (Handle)sortHandle );
		sortData = *sortHandle;
		
		//  Create a pixel buffer for the copland task.  It doesn't really have
		//  access to the memory that's in the GWorld...
		//pixelBuffer =  NewHandle( N );
		//HLockHi( (Handle)pixelBuffer );
		//pixelBufferData = *pixelBuffer;
	}

	return true;
}


void	SortPicts::CopyFromSortData( void )
{
	long			horiz;
	long			loop;
	SortPixel		pixel;
	
	horiz = 0;

	UseSortData();

	for( loop = 0; loop < N; ++loop)
	{
		pixel = sortData[loop];
		sortPixels[horiz] = pixel;
		//pixelBufferData[loop] = pixel;
		
		if( ++horiz == pictWidth)
		{
			horiz = 0;
			sortPixels += pictRowBytes;
		}
	}
	
	UnuseSortData();	
}


void	SortPicts::ReleaseSortData( void)
{
	DisposeHandle( (Handle) sortHandle );
	sortHandle = nil;
	
	//DisposeHandle( (Handle) pixelBuffer );
	//pixelBuffer = nil;
	//pixelBufferData = nil;
}


void	SortPicts::MakeSortData( void)
{
	long			horiz;
	long			loop;
	SortPixel		pixel;
	
	UseSortData();

	horiz = 0;

	for( loop = 0; loop < N; ++loop)
	{
		pixel = sortPixels[horiz];
		sortData[loop] = ((SortData) pixel) | (loop << 8);
		//pixelBufferData[loop] = pixel;
		
		if( ++horiz == pictWidth)
		{
			horiz = 0;
			sortPixels += pictRowBytes;
		}
	}

	UnuseSortData();	
}


void	SortPicts::InitializeSortWindowRect( ConstStr255Param pictName)
{
	Rect			**windPictRectResource;
	
	gNumWindowsCreated++;
	
	if( (gPrefsRef > 0) &&
		((windPictRectResource = (Rect **) GetNamedResource( 'RECT', pictName)) != NULL))
	{
			windPictRect = **windPictRectResource;
	}
	else
	{
		windPictRect = sortRect;
		windPictRect.top += kWindPictRectVoffset;
		windPictRect.bottom += kWindPictRectVoffset;
		
		OffsetRect( &windPictRect,
					(short) gNumWindowsCreated * 10, (short) gNumWindowsCreated * 10 + 30);
	}
}

void	SortPicts::OpenDefaultSortWindows( void)
{
	SortPicts	*newWindow;
	Str255		defaultWindowName;
	short		i;
	
	/*	Open additional windows for each of the Preferences Rects out there	*/
	i = 2;
	do
	{
		GetIndString( defaultWindowName, kDefaultWindowResource, i);
		if( defaultWindowName[0] == 0)
			break;
		
		newWindow = new SortPicts( defaultWindowName);
		
		i ++;
	} while( true);
	
	GetIndString( defaultWindowName, kDefaultWindowResource, 1);
	
	if( defaultWindowName[0] != 0)
	{
		::strcpy( (char *) sortPictsName, (char *) defaultWindowName);
		CreateWindow( kNormalWindow);
	}
	else
	{
		SelectRandomPicture();	
		CreateWindow( kNormalWindow);
	}
}

void	SortPicts::SelectRandomPicture( void)
{
	long		r;
	Str255		randomPictName;
	
	r = Random( gNumSortPictResources);
	GetMenuItemText( gPicturesMenu, (short) r, randomPictName);
	::strcpy( (char *) sortPictsName, (char *) randomPictName);
}
void	SortPicts::UseSortData( void)
{
	sortPixmap = GetGWorldPixMap( sortGWorld);

	LockPixels( sortPixmap);

	sortPixels = (SortPixelPtr) GetPixBaseAddr( sortPixmap);
	
	if ( !coplandTask )
	{
		HLock( (Handle) sortHandle);
		
		sortData = *sortHandle;
	}
}

void	SortPicts::UnuseSortData( void)
{
	if ( !coplandTask ) {
		HUnlock( (Handle) sortHandle);
	
		UnlockPixels( sortPixmap);
	}
}




/*
 *	RandomPixel
 *  -----------
 *
 *
 *	RandomPixel will return a random number, x, between 0  x < N
 *	where N is the number of pixels in the image.
 */
long	SortPicts::RandomPixel( void)		//	Return a # between 0 <= x < max
{
	return Random( N);
}

// need our oun random routine since we can't call the toolbox random routine from an MP task.

/*
 *	Random
 *	------
 *	Usage:		r = Random( num)
 *
 *	Random will return a random number, r, 0  r < num
 */
long	SortPicts::Random( long num)
{
	long			randomNum = 0x7FFFFFFF;		//	The ultimate long
	long			tempN;
	short			numNBits;
	short			numShiftBits, numRandomBits;
	
//	Random Number Initialization
	tempN = num;
	numNBits = 1;				// Using the additive conguential method (Random numbers)
	while (tempN >>= 1)			// Count the number of bits required to hold N	
		++numNBits;				//	example:  to hold 0x37 requires 6 bits (0-3F)

	numRandomBits = 31;	
	numShiftBits = numRandomBits - numNBits;
	
	if( randomSeed < 0)
		randomSeed = -randomSeed;
	
	while( randomNum >= num || randomNum < 0)
	{
//		RandomNum generator
		randomNum = randomSeed = (randomSeed & 1) ^ ((randomSeed & (1L << 28)) >> 28)
							   ? (randomSeed >> 1) : (randomSeed >> 1) | 0x40000000;
//		This next line "mixes" some of the upper bits with the lower bits
		randomNum = randomNum & (0xFFFFFFFF >> numShiftBits);
//		randomNum = (randomNum >> numShiftBits) ^ (randomNum & (0x7FFFFFFF >> numShiftBits));
	}
	
	return randomNum;	
}
void	SortPicts::Scramble( void)
{
	long			loop;
	
	if ( !coplandTask ) {
		UseSortData();
	}
	UseUpdateFunc( ::UpdateScramble);
	
	for( loop = 1; loop < N; loop ++)
		ExchangeSortItem( Random(loop), loop);
	
	if ( !coplandTask ) {
		UnuseSortData();
	}
}


void	SortPicts::UseUpdateFunc( UpdateFuncPtr func)
{
	updateFunc = func;
}

void	SortPicts::Update( void)
{
	(*updateFunc)( this);
}


void	UpdateScramble( SortPicts *sortPicts)
{
	sortPicts->UpdateScramble();
}


void	UpdateSortPict( SortPicts *sortPicts)
{
	sortPicts->UpdateScramble();
}


void	SortPicts::UpdateScramble( void)
{
	UpdateSortPict();
}

void	SortPicts::UpdateSortPict( void)
{
	GrafPtr			savePort;

	if ( !coplandTask )
	{
		if( copyBitsRect.bottom > copyBitsRect.top) {
			UseSortData();
		
			GetPort( &savePort);	
			SetPort( me);
			
			copyBitsRect.left = 0;
			copyBitsRect.right = (short) pictWidth;
			windPictRect.top = copyBitsRect.top +kWindPictRectVoffset;
			windPictRect.bottom = copyBitsRect.bottom +kWindPictRectVoffset;
		
			CopyBits(	(const BitMap *) *sortPixmap,
						(const BitMap *) (*(((CWindowPtr)me)->portPixMap)),
						&copyBitsRect, &copyBitsRect, 
						srcCopy, (RgnHandle)0);
			
			copyBitsRect.top = windPictRect.bottom;
			copyBitsRect.bottom = windPictRect.top;
			
			SetPort( savePort);
			
			UnuseSortData();
		}
	} else {
			//GetPort( &savePort);	
			//SetPort( me);
			if( copyBitsRect.bottom > copyBitsRect.top) {
				copyBitsRect.left = 0;
				copyBitsRect.right = (short) pictWidth;
				windPictRect.top = copyBitsRect.top +kWindPictRectVoffset;
				windPictRect.bottom = copyBitsRect.bottom +kWindPictRectVoffset;
		
				this->Draw();
				copyBitsRect.top = windPictRect.bottom;
				copyBitsRect.bottom = windPictRect.top;
			

			}
			//SetPort( savePort);
	}
}
extern short	gSortAlgorithm;

void	SortPicts::UseSortFunc( SortFuncPtr sort)
{
	UseUpdateFunc( ::UpdateSortPict);
	
	sortFunc = sort;
}

void	SortPicts::Sort( void)
{	
	short		sortAlgorithm;
	
	sortAlgorithm = gSortAlgorithm;
	if( sortAlgorithm == iRandomSort)
		sortAlgorithm = (short) Random( iRandomSort -2);

	if ( !coplandTask )
		UseSortData();
	
	switch (sortAlgorithm)
	{
		case	iQuickSort:
			UseSortFunc( ::QSort);
			break;
		case	iShellSort:
			UseSortFunc( ::ShellSort);
			break;
		case	iHeapSort:
			UseSortFunc( ::HeapSort);
			break;
		
		default:
			UseSortFunc( ::QSort);
			break;
	}
	(*sortFunc)( this);
	Update();
		
	if ( !coplandTask )
	{	
		UnuseSortData();
	}
	
}
